// standard OS includes (vxWorks)
#include <vxWorks.h>
#include <ctype.h>
#include <taskLib.h>
#include <stdio.h>
#include <stdlib.h>
#include <ioLib.h>
#include <dirent.h>

// MAME includes
#include "osdepend.h"
#include "driver.h"

//#define DIGITA_DEBUG
#ifdef DIGITA_DEBUG
static char	__message[256];
extern void DisplayText(char *);
void LogText( char *message	);

#define DEBUG_OUT(format, args...) \
( \
	sprintf(__message, format, ##args), \
	LogText(__message) \
)
#else
#define DEBUG_OUT(format, args...) 
#endif

// local prototypes
#ifdef MESS
void * messfront(void *);
#else
void * mamefront(void *);
#endif

// external prototypes
extern void detect_camera_type(void);
extern void eventHandlerTask(void);
extern void * mame (void * gamename);
extern unsigned long RGB2YCYC823( unsigned char r, unsigned char g, unsigned char b);
extern void update_screen(void);

extern void ui_displaymenu(const char **items,const char **subitems,char *flag,int selected,int arrowize_subitem);
extern void decompose_rom_sample_path (char *rompath, char *samplepath);

#define kREPEAT_TIME 7

#ifdef SOUND
extern void *DigitaSoundTask(void *dummy);
#endif

// external globals
extern struct osd_bitmap 	*scrbitmap;
extern int					camera_type;

// Globals
unsigned long 				*gLCD_buffer1 = 0;
unsigned long 				*gLCD_buffer2 = 0;
int 						gResolution = 0;
int 						gPadOrientation = 0;
int							mameTaskID = 0;
int 						soundTaskID = 0;
int							play_sound = 0;
int							cpu_slices_per_frame_override = 0;

/* empty list that will be replaced with _drivers[] when we load the game modules */
extern const struct GameDriver *_drivers[];
const struct GameDriver **drivers = _drivers;

/* The "root" driver, defined so we can have &driver_##NAME in macros. */
struct GameDriver driver_0 =
{
	__FILE__,
	0,
	"",
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	NOT_A_DRIVER
};


// locals Globals
static unsigned long 		*gLCD_buffer = 0;

void setup_application(void)
{
	int i;
	
	if (camera_type == kTypeMinolta1)
	{
	    GMInitGraf(NULL);
	    GMInitFonts();
	
		// allocate 2 lcd buffers
		MMNewAlignedPtr((180 * 287 * 2 * 4), 16, (char *) &gLCD_buffer);
	
		// clear the display
		gLCD_buffer1 = gLCD_buffer;
		for (i = 0;i < 180 * 287 * 2;i++)
		{
			*gLCD_buffer1++ = 0x10801080; // this is black in the YCC LCD world
		}
		
		// set the globals
		gLCD_buffer1 = gLCD_buffer;
		gLCD_buffer2 = gLCD_buffer + 180 * 287;
		
		// display the port
	    LMSetupBuffers(gLCD_buffer1, gLCD_buffer2, FALSE);
	    
		// turn on the LCD
	    LMEnableLCDController();
	
		MPUSendLens(1);
	}
	else if (camera_type == kTypeHP1)
	{
	    GMInitGraf(NULL);
	    GMInitFonts();
	    LMAllocateBuffers();
	    LMSwitchNormalMode();
	    LMSwitchLiveviewMode();
	    LMEnableLCDController();
	}
	else if (camera_type == kTypeKodak1)
	{
		// initialized the MPU and wait for it to finish initializing
		MPUInit();
		while( !smMconStat )
			taskDelay(20);

		// allocate 2 lcd buffers that are continuous so we can use them for interlaced
		// mode also
		MMNewAlignedPtr((288*216*4*2), 16, (char *)&gLCD_buffer);

		// clear the display
		gLCD_buffer1 = gLCD_buffer;
		for (i = 0;i < 288*216*2;i++)
		{
			*gLCD_buffer1++ = 0x80108010; // this is black in the YCC PPC 823 LCD world
		}
		
		// set the globals
		gLCD_buffer1 = gLCD_buffer;
		gLCD_buffer2 = gLCD_buffer + 288*216;
		
		// display the port
	    LMSetupBuffers(gLCD_buffer1, gLCD_buffer2, FALSE);
	    
		// turn on the LCD
	    LMEnableLCDController();

		// Disable the blinking green viewfinder LED.
		MPUSetLED( LED_VIEWFINDER | LED_OFF );
	}
	else if (camera_type == kTypeKodak2)
	{
		CMInit();
		HYInit();
		
		// allocate 2 lcd buffers that are continuous so we can use them for interlaced
		// mode also
		MMNewAlignedPtr((288*216*4*2), 16, (char *)&gLCD_buffer);

		// clear the display
		gLCD_buffer1 = gLCD_buffer;
		for (i = 0;i < 288*216*2;i++)
		{
			*gLCD_buffer1++ = 0x80108010; // this is black in the YCC PPC 823 LCD world
		}
		
		// set the globals
		gLCD_buffer1 = gLCD_buffer;
		gLCD_buffer2 = gLCD_buffer + 288*216;
		
		// display the port
	    LMSetupBuffers(gLCD_buffer1, gLCD_buffer2, FALSE);
	    
		// turn on the LCD
	    LMEnableLCDController();

		// Disable the blinking green viewfinder LED.
		MPUSetLED( LED_VIEWFINDER | LED_OFF );
	}
}

void fix_driver_table(void)
{
	struct GameDriver **walk_drivers_link;
	struct GameDriver **set_drivers_link;

	// remove gaps (zeros or unresolved references) in the game driver list,
	// cause MAME expects it that way.
	walk_drivers_link = drivers;
	set_drivers_link = drivers;
	while (*walk_drivers_link != (struct GameDriver *)0xFFEEDDCC)
	{
		while (*walk_drivers_link == 0)
			walk_drivers_link++;
		
		if (*walk_drivers_link != (struct GameDriver *)0xFFEEDDCC)
		{
			*set_drivers_link = *walk_drivers_link;
			set_drivers_link++;
			walk_drivers_link++;
		}
	}
	*set_drivers_link = 0;
}

void entry(void)
{
	fix_driver_table();
	
	detect_camera_type();

	setup_application();
	
#ifdef SOUND
	// Start up streaming audio thread...
	// sound task must have highest priority
	soundTaskID = taskSpawn("SOUND", 2, 0, 4096, 
		(FUNCPTR)DigitaSoundTask, NULL, 
		0, 0, 0, 0, 0, 0, 0, 0, 0);
#endif

	// start MAME thread
#ifdef MESS
	mameTaskID = taskSpawn("MESSD", 30, 0, 1024*64, 
		(FUNCPTR)messfront, NULL, 
		0, 0, 0, 0, 0, 0, 0, 0, 0);
#else
	mameTaskID = taskSpawn("MAMED", 30, 0, 1024*64, 
		(FUNCPTR)mamefront, NULL, 
		0, 0, 0, 0, 0, 0, 0, 0, 0);
#endif

	// this task now becomes the event handler task
	eventHandlerTask();
}

#ifdef MESS
int Validate_ROM(char * filename, const struct IODevice *dev)
{
	char extension[10];
	char * ext;
	
	/* find extension */
	ext = strrchr(filename, '.');
	if (ext)
	{
		ext++;
		strcpy(extension, ext);

		// delete spaces any from extension
		if (extension[0] == ' ')
		{
			extension[0] = 0;
		}
		else if (extension[1] == ' ')
		{
			extension[1] = 0;
		}
		else if (extension[2] == ' ')
		{
			extension[2] = 0;
		}

		while (dev->type != IO_END)
		{
			const char *dst = dev->file_extensions;
			/* scan supported extensions for this device */
			while (dst && *dst)
			{
				if (stricmp(dst,extension) == 0)
				{
                    return dev->type;
				}
				/* skip '\0' once in the list of extensions */
				dst += strlen(dst) + 1;
			}
			dev++;
		}
	}
	
	return -1;
}

int Select_ROM(char *dirpath, const struct IODevice *dev, char * rom_name)
{
    DIR     		*b;
    struct dirent 	*c;
	int 			sel;
	int 			menu_total;
	char 			**menu_item;
	int 			visible;
	int 			rom_type;
		
	// clear the screen
	osd_clearbitmap(scrbitmap);

	// first is default selected
	sel = 0;

	visible = (Machine->uiheight / (3 * Machine->uifontheight / 2))/2 - 1;

	// count number of files in rom directory
	menu_total = 0;
    b = opendir(dirpath);
    if (b) 
    {
    	
		while ((c = readdir(b)))
		{
			if (Validate_ROM(c->d_name, dev) != -1)
			{
				menu_total++;
			}
		}
		closedir(b);
	}

	// allocate enough space
	menu_item = (char **)malloc(sizeof(char **) * (menu_total + 2));

	menu_total = 0;
	
	// add first one for blank selection
	menu_item[menu_total++] = "";
	
    b = opendir(dirpath);
    if (b) 
    {
		while ((c = readdir(b))) 
		{
			rom_type = Validate_ROM(c->d_name, dev);
			if (rom_type != -1)
			{
				menu_item[menu_total] = malloc(strlen(c->d_name) + 1);
				strcpy(menu_item[menu_total], c->d_name);
				menu_total++;
			}
		}
		closedir(b);
	}

	// end of the list
	menu_item[menu_total] = 0;

	while (TRUE)
	{
		ui_displaymenu((const char **)menu_item,0,0,sel,0);
		taskDelay(7);
		update_screen();

		if (input_ui_pressed_repeat(IPT_UI_DOWN,kREPEAT_TIME))
			sel = sel + 1;

		if (input_ui_pressed_repeat(IPT_UI_UP,kREPEAT_TIME))
			sel = sel - 1;
			
		if (input_ui_pressed_repeat(IPT_UI_RIGHT,kREPEAT_TIME))
			sel = sel + visible - 1;

		if (input_ui_pressed_repeat(IPT_UI_LEFT,kREPEAT_TIME))
			sel = sel - visible + 1;

		if (sel > menu_total - 1)
		{
			sel = menu_total - 1;
		}
		else if (sel < 0)
		{
			sel = 0;
		}
		
		if (input_ui_pressed(IPT_UI_SELECT))
		{
			break;
		}
		
		if (input_ui_pressed(IPT_UI_CANCEL))
		{
			sel = -1;
			break;
		}
	}

	if (sel != -1)
	{
		// save a copy in return string
		strcpy(rom_name, menu_item[sel]);
	}
	
	// free all the strings
	// skip the first one since it's hard coded
	menu_total = 1;
	while(menu_item[menu_total])
	{
		free(menu_item[menu_total]);
		menu_total++;
	}

	// free the table
	free(menu_item);
	
	if (sel != -1)
	{
		return Validate_ROM(rom_name, dev);
	}
	else
	{
		return -2;
	}
}

char rom_files[MAX_IMAGES][20];

void * messfront(void * dummy)
{
	int sel;
	int visible;
	int menu_total;
	char **menu_item;
	int i;
	int index;

	memset(&options, 0, sizeof(options));
	// Init default options
	options.norotate     = 0;
	options.ror          = 0;
	options.rol          = 0;
	options.flipx        = 0;
	options.flipy        = 0;
	
	options.use_artwork  = 1;
	options.image_count  = 0;
	options.color_depth  = 8; /* force 8-bit color */
	
	if ((camera_type == kTypeMinolta1) || 
		(camera_type == kTypeHP1))
	{
		options.use_samples  = 0;
		options.samplerate   = 0;
		play_sound = 0;
	}
	else
	{
		options.use_samples  = 1;
		options.samplerate   = 11025;
		play_sound = 1;
	}
	
	gResolution = 0;
	gPadOrientation = 0;
	cpu_slices_per_frame_override = 1;
	
	// init MAME's search directories for ROMs
	decompose_rom_sample_path("/B/SYSTEM/ROMS;/B/SYSTEM/", 
		"/B/SYSTEM/SAMPLES;/B/SYSTEM/");

	// first is default selected
	sel = 0;
	
	// count drivers so we can allocate enough memory
	menu_total = 0;
	for (i = 0;drivers[i];i++)
	{
		if (drivers[i])
		{
			// check if rom exists and add it to list
			if (osd_faccess (drivers[i]->name, OSD_FILETYPE_ROM))
			{
				menu_total++;
			}
		}
	}
	menu_item = (char **)malloc(sizeof(char **) * (menu_total + 1));
	
	// reset count to start again and this time add the ones we find
	menu_total = 0;
	for (i = 0;drivers[i];i++)
	{
		if (drivers[i])
		{
			// check if rom exists and add it to list
			if (osd_faccess (drivers[i]->name, OSD_FILETYPE_ROM))
			{
				menu_item[menu_total++] = (char *)drivers[i]->name;
			}
		}
	}
	
	// end of the list
	menu_item[menu_total] = 0;

	while(TRUE)
	{
		struct GameOptions saveOptions;
		int save_play_sound;
		int saveResolution;
		int savePadOrientation;
		struct MachineDriver fake_driver;
		struct GameDriver fake_gamedrv;
		struct InputPort fake_input_port;

		// reset screen to normal orientation and resolution while in main menu
		// we will restore it just before we start the game		

		memset(&fake_driver, 0, sizeof(fake_driver));
		memset(&fake_gamedrv, 0, sizeof(fake_gamedrv));
		memset(&fake_input_port, 0, sizeof(fake_input_port));
		
		saveOptions = options;
		save_play_sound = play_sound;
		saveResolution = gResolution;
		savePadOrientation = gPadOrientation;
		
		options.norotate    = 0;
		options.ror         = 0;
		options.rol         = 0;
		options.flipx       = 0;
		options.flipy       = 0;
		options.samplerate  = 0;
		options.use_samples = 0;
		options.use_artwork = 0;
		options.color_depth  = 8; /* force 8-bit color */
		options.image_count = 0;
		
		play_sound = 0;
		gResolution = 0;
		gPadOrientation = 0;

		// Init enough of MAME's structures to get the user menus to work		
		
		Machine->drv = &fake_driver;
		Machine->drv->frames_per_second = 60;
		Machine->drv->video_attributes = 0;
		Machine->drv->screen_width = 288;
		Machine->drv->screen_height = 216;
		Machine->drv->color_table_len = 0;
		
		Machine->gamedrv = &fake_gamedrv;
		
		Machine->orientation = ROT0;
		Machine->ui_orientation = ROT0;

		Machine->input_ports_default = &fake_input_port;
		Machine->input_ports = &fake_input_port;
		fake_input_port.type = IPT_END;
		
		code_init();
		
		if ((camera_type == kTypeHP1) || (camera_type == kTypeMinolta1))
		{
			Machine->scrbitmap = osd_create_display(360,240,8,0);
		}
		else if ((camera_type == kTypeKodak1) || (camera_type == kTypeKodak2))
		{
			Machine->scrbitmap = osd_create_display(288,216,8,0);
		}

		Machine->uifont = builduifont();

		visible = (Machine->uiheight / (3 * Machine->uifontheight / 2))/2 - 1;

		osd_allocate_colors(0,0,0,0);
		
		while (TRUE)
		{
			ui_displaymenu((const char **)menu_item,0,0,sel,0);
			taskDelay(7);
			update_screen();

			if (input_ui_pressed(IPT_UI_CONFIGURE))
			{
				// my little options menu
				const char *game_submenu_item[20];
				int game_submenu_total;
				const char *game_menu_item[20];
				int game_menu_total;
				int gamesel;
				
				osd_clearbitmap(scrbitmap);

				gamesel = 0;
				
				while (TRUE)
				{
					game_menu_total = 0;
					game_submenu_total = 0;
					
					game_menu_item[game_menu_total++] = "Pad Orientation"; 
					if (savePadOrientation == 0) 
						game_submenu_item[game_submenu_total++] = "Up";
					else if (savePadOrientation == 1)
						game_submenu_item[game_submenu_total++] = "Right";
					else if (savePadOrientation == 2)
						game_submenu_item[game_submenu_total++] = "Down";
					else if (savePadOrientation == 3)
						game_submenu_item[game_submenu_total++] = "Left";
					else
						game_submenu_item[game_submenu_total++] = "Unknown";

					game_menu_item[game_menu_total++] = "Resolution";
					if (saveResolution > 0) 
						game_submenu_item[game_submenu_total++] = "High";
					else  if (saveResolution < 0) 
						game_submenu_item[game_submenu_total++] = "Low";
					else 
						game_submenu_item[game_submenu_total++] = "Med";
					
					game_menu_item[game_menu_total++] = "No Rotation";
					if (saveOptions.norotate) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
					game_menu_item[game_menu_total++] = "Rotation Right"; 
					if (saveOptions.ror) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
					game_menu_item[game_menu_total++] = "Rotation Left"; 
					if (saveOptions.rol) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
					game_menu_item[game_menu_total++] = "Flip X"; 
					if (saveOptions.flipx) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
					game_menu_item[game_menu_total++] = "Flip Y"; 
					if (saveOptions.flipy) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
					
					game_menu_item[game_menu_total++] = "Sound";
					if (saveOptions.samplerate) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 

					game_menu_item[game_menu_total++] = "Samples";
					if (saveOptions.use_samples) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 

					game_menu_item[game_menu_total++] = "Fake Sound";
					if (!save_play_sound) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
						
					game_menu_item[game_menu_total++] = "Artwork";
					if (saveOptions.use_artwork) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 

					game_menu_item[game_menu_total++] = "CPU Slice Speedup";
					if (cpu_slices_per_frame_override) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 

					game_menu_item[game_menu_total] = 0;
					game_submenu_item[game_submenu_total] = 0;
					
					ui_displaymenu((const char **)game_menu_item,(const char **)game_submenu_item,0,gamesel,0);
					taskDelay(7);
					update_screen();

					if (input_ui_pressed_repeat(IPT_UI_DOWN,kREPEAT_TIME))
						gamesel = gamesel + 1;

					if (input_ui_pressed_repeat(IPT_UI_UP,kREPEAT_TIME))
						gamesel = gamesel - 1;
						
					if (gamesel > game_menu_total - 1)
					{
						gamesel = game_menu_total - 1;
					}
					else if (gamesel < 0)
					{
						gamesel = 0;
					}

					if (input_ui_pressed(IPT_UI_SELECT))
					{
						switch (gamesel)
						{
							case 0:
							{
								if (savePadOrientation == 0)
									savePadOrientation = 1;
								else if (savePadOrientation == 1)
									savePadOrientation = 2;
								else if (savePadOrientation == 2)
									savePadOrientation = 3;
								else if (savePadOrientation == 3)
									savePadOrientation = 0;
								else
									savePadOrientation = 0;
								break;
							}
							case 1:
							{
								if (saveResolution < 0)
									saveResolution = 0;
								else if (saveResolution == 0)
									saveResolution = 1;
								else
									saveResolution = -1;
								break;
							}
							case 2:
							{
								if (saveOptions.norotate)
									saveOptions.norotate = 0;
								else
									saveOptions.norotate = 1;
								break;
							}
							case 3:
							{
								if (saveOptions.ror)
									saveOptions.ror = 0;
								else
									saveOptions.ror = 1;
								break;
							}
							case 4:
							{
								if (saveOptions.rol)
									saveOptions.rol = 0;
								else
									saveOptions.rol = 1;
								break;
							}
							case 5:
							{
								if (saveOptions.flipx)
									saveOptions.flipx = 0;
								else
									saveOptions.flipx = 1;
								break;
							}
							case 6:
							{
								if (saveOptions.flipy)
									saveOptions.flipy = 0;
								else
									saveOptions.flipy = 1;
								break;
							}
							case 7:
							{
								if (saveOptions.samplerate)
									saveOptions.samplerate = 0;
								else
									saveOptions.samplerate = 11025;
								break;
							}
							case 8:
							{
								if (saveOptions.use_samples)
									saveOptions.use_samples = 0;
								else
									saveOptions.use_samples = 1;
								break;
							}
							case 9:
							{
								
								if (save_play_sound)
									save_play_sound = 0;
								else
									if ((camera_type == kTypeMinolta1) || (camera_type == kTypeHP1))
										save_play_sound = 0;
									else
										save_play_sound = 1;
								break;
							}
							case 10:
							{
								if (saveOptions.use_artwork)
									saveOptions.use_artwork = 0;
								else
									saveOptions.use_artwork = 1;
								break;
							}
							
							case 11:
							{
								if (cpu_slices_per_frame_override)
									cpu_slices_per_frame_override = 0;
								else
									cpu_slices_per_frame_override = 1;
								break;
							}
							
							default:
							{
								break;
							}
						}
						osd_clearbitmap(scrbitmap);
					}
					
					if (input_ui_pressed(IPT_UI_CANCEL) 
						|| input_ui_pressed(IPT_UI_CONFIGURE))
					{
						osd_clearbitmap(scrbitmap);
						break;
					}
				}
			}
			
			if (input_ui_pressed_repeat(IPT_UI_DOWN,kREPEAT_TIME))
				sel = sel + 1;

			if (input_ui_pressed_repeat(IPT_UI_UP,kREPEAT_TIME))
				sel = sel - 1;
				
			if (input_ui_pressed_repeat(IPT_UI_RIGHT,kREPEAT_TIME))
				sel = sel + visible - 1;

			if (input_ui_pressed_repeat(IPT_UI_LEFT,kREPEAT_TIME))
				sel = sel - visible + 1;

			if (sel > menu_total - 1)
			{
				sel = menu_total - 1;
			}
			else if (sel < 0)
			{
				sel = 0;
			}
			
			if (input_ui_pressed(IPT_UI_SELECT))
			{
				break;
			}
		}

		// find the driver in the list
		for (i = 0;drivers[i];i++)
		{
			if (drivers[i])
			{
				if (drivers[i]->name == menu_item[sel])
				{
					break;
				}
			}
		}

		// add roms until user selects blank item from menu

		saveOptions.image_count = 0;
		for (index = 0; index < MAX_IMAGES; index++)
		{
			saveOptions.image_files[index].name = 0;
			saveOptions.image_files[index].type = IO_END;
		}
		
		while(saveOptions.image_count < MAX_IMAGES)
		{
			saveOptions.image_files[saveOptions.image_count].type 
				= Select_ROM("/B/SYSTEM/ROMS/", 
					drivers[i]->dev,
					rom_files[saveOptions.image_count]);
			saveOptions.image_files[saveOptions.image_count].name 
				= rom_files[saveOptions.image_count];
			if (stricmp(saveOptions.image_files[saveOptions.image_count].name,"") != 0)
			{
				saveOptions.image_count++;
			}
			else
			{
				saveOptions.image_files[saveOptions.image_count].name = 0;
				saveOptions.image_files[saveOptions.image_count].type = 0;
				break;
			}
		}

		// clear the screen
		osd_clearbitmap(scrbitmap);
		update_screen();
		update_screen(); /* clear both buffers in case we use interlaced mode */

		osd_close_display();
		Machine->scrbitmap = 0;
		
		freegfx(Machine->uifont);
		Machine->uifont = 0;

		Machine->gamedrv = 0;
		Machine->drv = 0;
		
		// restore current selected option settings
		gResolution = saveResolution;
		gPadOrientation = savePadOrientation;
		options = saveOptions;
		play_sound = save_play_sound;

		code_close();

		// start the game
		mame( menu_item[sel] );
	}
	
	free(menu_item);
	
	return 0;
}

#else

void * mamefront(void * dummy)
{
	int sel;
	int visible;
	int menu_total;
	char **menu_item;
	int i;

	memset(&options, 0, sizeof(options));
	// Init default options
	options.norotate     = 0;
	options.ror          = 0;
	options.rol          = 0;
	options.flipx        = 0;
	options.flipy        = 0;
	
	options.use_artwork  = 1;
	options.color_depth  = 8; /* force 8-bit color */
	
	if ((camera_type == kTypeMinolta1) || 
		(camera_type == kTypeHP1))
	{
		options.use_samples  = 0;
		options.samplerate   = 0;
		play_sound = 0;
	}
	else
	{
		options.use_samples  = 1;
		options.samplerate   = 11025;
		play_sound = 1;
	}
	
	gResolution = 0;
	gPadOrientation = 0;
	cpu_slices_per_frame_override = 1;

	// init MAME's search directories for ROMs
	decompose_rom_sample_path("/B/SYSTEM/ROMS;/B/SYSTEM/", 
		"/B/SYSTEM/SAMPLES;/B/SYSTEM/");

	// first is default selected
	sel = 0;
	
	// count drivers so we can allocate enough memory
	menu_total = 0;
	for (i = 0;drivers[i];i++)
	{
		if (drivers[i])
		{
			// check if rom exists and add it to list
			if (osd_faccess (drivers[i]->name, OSD_FILETYPE_ROM))
			{
				menu_total++;
			}
		}
	}
	menu_item = (char **)malloc(sizeof(char **) * (menu_total + 1));
	
	// reset count to start again and this time add the ones we find
	menu_total = 0;
	for (i = 0;drivers[i];i++)
	{
		if (drivers[i])
		{
			// check if rom exists and add it to list
			if (osd_faccess (drivers[i]->name, OSD_FILETYPE_ROM))
			{
				menu_item[menu_total++] = (char *)drivers[i]->name;
			}
		}
	}
	
	// end of the list
	menu_item[menu_total] = 0;

	while(TRUE)
	{
		struct GameOptions saveOptions;
		int save_play_sound;
		int saveResolution;
		int savePadOrientation;
		struct MachineDriver fake_driver;
		struct GameDriver fake_gamedrv;
		struct InputPort fake_input_port;

		// reset screen to normal orientation and resolution while in main menu
		// we will restore it just before we start the game		

		memset(&fake_driver, 0, sizeof(fake_driver));
		memset(&fake_gamedrv, 0, sizeof(fake_gamedrv));
		memset(&fake_input_port, 0, sizeof(fake_input_port));
		
		saveOptions = options;
		save_play_sound = play_sound;
		saveResolution = gResolution;
		savePadOrientation = gPadOrientation;
		
		options.norotate    = 0;
		options.ror         = 0;
		options.rol         = 0;
		options.flipx       = 0;
		options.flipy       = 0;
		options.samplerate  = 0;
		options.use_samples = 0;
		options.use_artwork = 0;
		options.color_depth  = 8; /* force 8-bit color */

		play_sound = 1;
		gResolution = 0;
		gPadOrientation = 0;

		// Init enough of MAME's structures to get the user menus to work		
		
		Machine->drv = &fake_driver;
		Machine->drv->frames_per_second = 60;
		Machine->drv->video_attributes = 0;
		Machine->drv->screen_width = 288;
		Machine->drv->screen_height = 216;
		Machine->drv->color_table_len = 0;

		Machine->gamedrv = &fake_gamedrv;
		
		Machine->orientation = ROT0;
		Machine->ui_orientation = ROT0;

		Machine->input_ports_default = &fake_input_port;
		Machine->input_ports = &fake_input_port;
		fake_input_port.type = IPT_END;
		
		code_init();
		
		if ((camera_type == kTypeHP1) || (camera_type == kTypeMinolta1))
		{
			Machine->scrbitmap = osd_create_display(360,240,8,0);
		}
		else if ((camera_type == kTypeKodak1) || (camera_type == kTypeKodak2))
		{
			Machine->scrbitmap = osd_create_display(288,216,8,0);
		}
		
		Machine->uifont = builduifont();
		
		visible = (Machine->uiheight / (3 * Machine->uifontheight / 2))/2 - 1;

		osd_allocate_colors(0,0,0,0);
		
		while (TRUE)
		{
			ui_displaymenu((const char **)menu_item,0,0,sel,0);
			taskDelay(7);
			update_screen();

			if (input_ui_pressed(IPT_UI_CONFIGURE))
			{
				// my little options menu
				const char *game_submenu_item[20];
				int game_submenu_total;
				const char *game_menu_item[20];
				int game_menu_total;
				int gamesel;
				
				osd_clearbitmap(scrbitmap);

				gamesel = 0;
				
				while (TRUE)
				{
					game_menu_total = 0;
					game_submenu_total = 0;
					
					game_menu_item[game_menu_total++] = "Pad Orientation"; 
					if (savePadOrientation == 0) 
						game_submenu_item[game_submenu_total++] = "Up";
					else if (savePadOrientation == 1)
						game_submenu_item[game_submenu_total++] = "Right";
					else if (savePadOrientation == 2)
						game_submenu_item[game_submenu_total++] = "Down";
					else if (savePadOrientation == 3)
						game_submenu_item[game_submenu_total++] = "Left";
					else
						game_submenu_item[game_submenu_total++] = "Unknown";

					game_menu_item[game_menu_total++] = "Resolution";
					if (saveResolution > 0) 
						game_submenu_item[game_submenu_total++] = "High";
					else  if (saveResolution < 0) 
						game_submenu_item[game_submenu_total++] = "Low";
					else 
						game_submenu_item[game_submenu_total++] = "Med";
					
					game_menu_item[game_menu_total++] = "No Rotation";
					if (saveOptions.norotate) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
					game_menu_item[game_menu_total++] = "Rotation Right"; 
					if (saveOptions.ror) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
					game_menu_item[game_menu_total++] = "Rotation Left"; 
					if (saveOptions.rol) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
					game_menu_item[game_menu_total++] = "Flip X"; 
					if (saveOptions.flipx) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
					game_menu_item[game_menu_total++] = "Flip Y"; 
					if (saveOptions.flipy) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
					
					game_menu_item[game_menu_total++] = "Sound";
					if (saveOptions.samplerate) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 

					game_menu_item[game_menu_total++] = "Samples";
					if (saveOptions.use_samples) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 

					game_menu_item[game_menu_total++] = "Fake Sound";
					if (!save_play_sound) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 
						
					game_menu_item[game_menu_total++] = "Artwork";
					if (saveOptions.use_artwork) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 

					game_menu_item[game_menu_total++] = "CPU Slice Speedup";
					if (cpu_slices_per_frame_override) 
						game_submenu_item[game_submenu_total++] = "Yes";
					else 
						game_submenu_item[game_submenu_total++] = "No"; 

					game_menu_item[game_menu_total] = 0;
					game_submenu_item[game_submenu_total] = 0;
					
					ui_displaymenu((const char **)game_menu_item,(const char **)game_submenu_item,0,gamesel,0);
					taskDelay(7);
					update_screen();

					if (input_ui_pressed_repeat(IPT_UI_DOWN,kREPEAT_TIME))
						gamesel = gamesel + 1;

					if (input_ui_pressed_repeat(IPT_UI_UP,kREPEAT_TIME))
						gamesel = gamesel - 1;
						
					if (gamesel > game_menu_total - 1)
					{
						gamesel = game_menu_total - 1;
					}
					else if (gamesel < 0)
					{
						gamesel = 0;
					}

					if (input_ui_pressed(IPT_UI_SELECT))
					{
						switch (gamesel)
						{
							case 0:
							{
								if (savePadOrientation == 0)
									savePadOrientation = 1;
								else if (savePadOrientation == 1)
									savePadOrientation = 2;
								else if (savePadOrientation == 2)
									savePadOrientation = 3;
								else if (savePadOrientation == 3)
									savePadOrientation = 0;
								else
									savePadOrientation = 0;
								break;
							}
							case 1:
							{
								if (saveResolution < 0)
									saveResolution = 0;
								else if (saveResolution == 0)
									saveResolution = 1;
								else
									saveResolution = -1;
								break;
							}
							case 2:
							{
								if (saveOptions.norotate)
									saveOptions.norotate = 0;
								else
									saveOptions.norotate = 1;
								break;
							}
							case 3:
							{
								if (saveOptions.ror)
									saveOptions.ror = 0;
								else
									saveOptions.ror = 1;
								break;
							}
							case 4:
							{
								if (saveOptions.rol)
									saveOptions.rol = 0;
								else
									saveOptions.rol = 1;
								break;
							}
							case 5:
							{
								if (saveOptions.flipx)
									saveOptions.flipx = 0;
								else
									saveOptions.flipx = 1;
								break;
							}
							case 6:
							{
								if (saveOptions.flipy)
									saveOptions.flipy = 0;
								else
									saveOptions.flipy = 1;
								break;
							}
							case 7:
							{
								if (saveOptions.samplerate)
									saveOptions.samplerate = 0;
								else
									saveOptions.samplerate = 11025;
								break;
							}
							case 8:
							{
								if (saveOptions.use_samples)
									saveOptions.use_samples = 0;
								else
									saveOptions.use_samples = 1;
								break;
							}
							case 9:
							{
								if (save_play_sound)
									save_play_sound = 0;
								else
									if ((camera_type == kTypeMinolta1) || (camera_type == kTypeHP1))
										save_play_sound = 0;
									else
										save_play_sound = 1;
								break;
							}
							case 10:
							{
								if (saveOptions.use_artwork)
									saveOptions.use_artwork = 0;
								else
									saveOptions.use_artwork = 1;
								break;
							}
							
							case 11:
							{
								if (cpu_slices_per_frame_override)
									cpu_slices_per_frame_override = 0;
								else
									cpu_slices_per_frame_override = 1;
								break;
							}
							
							default:
							{
								break;
							}
						}
						osd_clearbitmap(scrbitmap);
					}
					
					if (input_ui_pressed(IPT_UI_CANCEL) 
						|| input_ui_pressed(IPT_UI_CONFIGURE))
					{
						osd_clearbitmap(scrbitmap);
						break;
					}
				}
			}
			
			if (input_ui_pressed_repeat(IPT_UI_DOWN,kREPEAT_TIME))
				sel = sel + 1;

			if (input_ui_pressed_repeat(IPT_UI_UP,kREPEAT_TIME))
				sel = sel - 1;
				
			if (input_ui_pressed_repeat(IPT_UI_RIGHT,kREPEAT_TIME))
				sel = sel + visible - 1;

			if (input_ui_pressed_repeat(IPT_UI_LEFT,kREPEAT_TIME))
				sel = sel - visible + 1;

			if (sel > menu_total - 1)
			{
				sel = menu_total - 1;
			}
			else if (sel < 0)
			{
				sel = 0;
			}
			
			if (input_ui_pressed(IPT_UI_SELECT))
			{
				break;
			}
		}

		// clear the screen
		osd_clearbitmap(scrbitmap);
		update_screen();
		update_screen(); /* clear both buffers in case we use interlaced mode */
	
		osd_close_display();
		Machine->scrbitmap = 0;
		
		freegfx(Machine->uifont);
		Machine->uifont = 0;

		Machine->gamedrv = 0;
		Machine->drv = 0;
		
		// restore current selected option settings
		gResolution = saveResolution;
		gPadOrientation = savePadOrientation;
		options = saveOptions;
		play_sound = save_play_sound;
		
		code_close();

		// start the game
		mame( menu_item[sel] );
	}
	
	free(menu_item);

	return 0;
}
#endif

/* update and display message log */
#define strLogFileName			"/B/MAME.LOG"
char *gStrLogFileName = NULL;

/* message to be added to the displayed log */
void LogText( char *message	)
{
	FILE *log;
	if (gStrLogFileName == NULL)
	{
		gStrLogFileName = strLogFileName;
	}
	log = fopen(gStrLogFileName, "a");
	if( log ) 
	{
		fprintf(log, "%s\r\n", message);
		fclose(log);
	}
} /* end of LogText() */

/* not a standard ANSI function but MAME uses it so stick it here */
int stricmp(const char *inString1, const char *inString2)
{
    char c1, c2;
    while (1)
    {
    	if (isupper(*inString1))
			c1 = tolower(*inString1);
		else
			c1 = *inString1;
		inString1++;
    	if (isupper(*inString2))
    		c2 = tolower(*inString2);
    	else
    		c2 = *inString2;
    	inString2++;
        if (c1 < c2) return -1;
        if (c1 > c2) return 1;
        if (c1 == 0) return 0;
    }
}
