/* vxWorks includes */
#include <vxWorks.h>
#include <symLib.h>
#include <sysSymTbl.h>
#include <loadLib.h>
#include <unldLib.h>
#include <dirent.h>
#include <stdio.h>
#include <ioLib.h>
#include <strLib.h>

/* MAME includes */
#include "driver.h"

void Load_Directory(char *dirpath)
{
    DIR     		*b;
    struct dirent 	*c;
	char    		fname[100];
	int 			fid;
	
    b = opendir(dirpath);
    if (b) 
    {
		while ((c = readdir(b))) 
		{
			strcpy(fname, dirpath);
			strcat(fname, c->d_name);
			fid = open(fname, O_RDONLY, 0 );
			loadModule(fid, LOAD_ALL_SYMBOLS);
			close(fid);
		}
		closedir(b);
	}
}

void entry(void)
{
	struct GameDriver **drivers_link;
	struct GameDriver *_drivers_link;
	struct snd_interface **sndintf_link;
	struct snd_interface *_sndintf_link;
	struct cpu_interface **cpuintf_link;
	struct cpu_interface *_cpuintf_link;

	void * entry_link;
	void * temp = 0;

	int fid;
	SYM_TYPE type;

	// Digita deleted the system symbol table at boot to save RAM 
	// so we have to recreate it here
	// so we can use it to link our stuff together at runtime.
	// of course all symbols in this part of the application will not
	// be in the symbol table so we cannot link anything here with
	// the rest of the code unless we do it manually.

	if (CreateSymtbl)
	{
		temp = malloc(1024*400);
		CreateSymtbl(temp);
	}

	// Load the main application including all CPU and SOUND emulators
	fid = open("/B/SYSTEM/MESS/MESS.O", O_RDONLY, 0 );
	loadModule(fid, LOAD_ALL_SYMBOLS);
	close(fid);

	// load sound drivers
	Load_Directory("/B/SYSTEM/MESS/OBJ/SOUND/");

	// load cpu drivers
	Load_Directory("/B/SYSTEM/MESS/OBJ/CPU/");

	// load sound driver table
	fid = open("/B/SYSTEM/MESS/OBJ/SNDINTRF.O", O_RDONLY, 0 );
	loadModule(fid, LOAD_ALL_SYMBOLS);
	close(fid);	
	
	// Update sound drivers table pointer
	// to point to actual sound driver table that is now fully linked
	// symbol table will return the most recent value
	symFindByName(sysSymTbl, "sndintf", (char **)&sndintf_link, &type);
	symFindByName(sysSymTbl, "xsndintf", (char **)&_sndintf_link, &type);
	*sndintf_link = _sndintf_link;

	// load cpu drivers table
	fid = open("/B/SYSTEM/MESS/OBJ/CPUTBL.O", O_RDONLY, 0 );
	loadModule(fid, LOAD_ALL_SYMBOLS);
	close(fid);

	// Update sound drivers table pointer
	// to point to actual sound driver table that is now fully linked
	// symbol table will return the most recent value
	symFindByName(sysSymTbl, "cpuintf", (char **)&cpuintf_link, &type);
	symFindByName(sysSymTbl, "_cpuintf", (char **)&_cpuintf_link, &type);
	*cpuintf_link = _cpuintf_link;

	// Load the game drivers, it will link with CPU and SOUND emulators
	Load_Directory("/B/SYSTEM/MESS/OBJ/SYSTEM/");

	// Load the game driver table, it will link with the drivers 
	// and CPU and SOUND emulators
	fid = open("/B/SYSTEM/MESS/OBJ/SYSTEM.O", O_RDONLY, 0 );
	loadModule(fid, LOAD_ALL_SYMBOLS);
	close(fid);

	// Update drivers table pointer
	// to point to actual driver table that is now fully linked
	// symbol table will return the most recent value
	symFindByName(sysSymTbl, "drivers", (char **)&drivers_link, &type);
	symFindByName(sysSymTbl, "_drivers", (char **)&_drivers_link, &type);
	*drivers_link = _drivers_link;

	// Find the entry point of the main program
	symFindByName(sysSymTbl, "entry", (char **)&entry_link, &type);

	// We are done with the system symbol table now 
	// so we can delete it and free up a bit more RAM for MAME games
	if (EmptySymtbl)
	{
		EmptySymtbl();
		free(temp);
	}
	
	// OK call the main program
	(*((void (*)(void))(entry_link)))();
}
